@@ -1,6 +1,18 @@
-"""Aioreactive module"""
+"""Aioreactive module.
 
-from typing import AsyncIterable, Awaitable, Callable, Iterable, Tuple, TypeVar, Union
+Contains the AsyncRx chained obserable that allows method chaining of all operators.
+
+Also contains all operators as plain functions.
+
+To use this module:
+
+Example:
+    >>> import aioreactive as rx
+    >>> xs = rx.from_iterable([1, 2, 3])
+    >>> ...
+
+"""
+from typing import Any, AsyncIterable, Awaitable, Callable, Iterable, Tuple, TypeVar, Union
 
 from expression.core import Option, pipe
 from expression.system.disposable import AsyncDisposable
@@ -28,9 +40,61 @@ class AsyncRx(AsyncObservable[TSource]):
     """
 
     def __init__(self, source: AsyncObservable[TSource]) -> None:
-        super().__init__()
         self._source = source
 
+    async def subscribe_async(self, observer: AsyncObserver[TSource]) -> AsyncDisposable:
+        """Subscribe to the async observable.
+
+        Uses the given observer to subscribe asynchronously to the async
+        observable.
+
+        Args:
+            observer: The async observer to subscribe.
+
+        Returns:
+            An async disposable that can be used to dispose the
+            subscription.
+        """
+        return await self._source.subscribe_async(observer)
+
+    def __getitem__(self, key: Union[slice, int]) -> "AsyncRx[TSource]":
+        """Slices the given source stream using Python slice notation.
+        The arguments to slice is start, stop and step given within
+        brackets [] and separated with the ':' character. It is
+        basically a wrapper around the operators skip(), skip_last(),
+        take(), take_last() and filter().
+
+        This marble diagram helps you remember how slices works with
+        streams. Positive numbers is relative to the start of the
+        events, while negative numbers are relative to the end
+        (on_completed) of the stream.
+
+         r---e---a---c---t---i---v---e---|
+         0   1   2   3   4   5   6   7   8
+        -8  -7  -6  -5  -4  -3  -2  -1
+
+        Example:
+        >>> result = source[1:10]
+        >>> result = source[1:-2]
+        >>> result = source[1:-1:2]
+
+        Args:
+            self: Source to slice
+            key: A slice object
+
+        Returns:
+            The sliced source stream.
+        """
+
+        from .filtering import slice as _slice
+
+        if isinstance(key, slice):
+            start, stop, step = key.start, key.stop, key.step
+        else:
+            start, stop, step = key, key + 1, 1
+
+        return AsyncRx(pipe(self, _slice(start, stop, step)))
+
     @classmethod
     def create(cls, source: AsyncObservable[TSource]) -> "AsyncRx[TSource]":
         """Create `AsyncChainedObservable`.
@@ -40,56 +104,13 @@ class AsyncRx(AsyncObservable[TSource]):
         """
         return cls(source)
 
-    async def subscribe_async(self, observer: AsyncObserver[TSource]) -> AsyncDisposable:
-        return await self._source.subscribe_async(observer)
-
-    def __getitem__(self, key: Union[slice, int]) -> "AsyncRx[TSource]":
-        """Slices the given source stream using Python slice notation.
-         The arguments to slice is start, stop and step given within
-         brackets [] and separated with the ':' character. It is
-         basically a wrapper around the operators skip(), skip_last(),
-         take(), take_last() and filter().
-
-         This marble diagram helps you remember how slices works with
-         streams. Positive numbers is relative to the start of the
-         events, while negative numbers are relative to the end
-         (on_completed) of the stream.
-
-         r---e---a---c---t---i---v---e---|
-         0   1   2   3   4   5   6   7   8
-        -8  -7  -6  -5  -4  -3  -2  -1
-
-         Example:
-         result = source[1:10]
-         result = source[1:-2]
-         result = source[1:-1:2]
-
-         Keyword arguments:
-         self -- Source to slice
-         key -- Slice object
-
-         Returne a sliced source stream."""
-
-        from .filter import slice as _slice
-
-        if isinstance(key, slice):
-            start, stop, step = key.start, key.stop, key.step
-        else:
-            start, stop, step = key, key + 1, 1
-
-        return AsyncRx(pipe(self, _slice(start, stop, step)))
+    @classmethod
+    def empty(cls) -> "AsyncRx[TSource]":
+        return AsyncRx(empty())
 
     @classmethod
     def from_iterable(cls, iter: Iterable[TSource]) -> "AsyncRx[TSource]":
-        from .create import of_seq
-
-        return AsyncRx(of_seq(iter))
-
-    @classmethod
-    def empty(cls) -> "AsyncRx[TSource]":
-        from .create import empty
-
-        return AsyncRx(empty())
+        return AsyncRx(from_iterable(iter))
 
     @classmethod
     def single(cls, value: TSource) -> "AsyncRx[TSource]":
@@ -97,10 +118,45 @@ class AsyncRx(AsyncObservable[TSource]):
 
         return AsyncRx(single(value))
 
+    def as_async_observable(self) -> AsyncObservable[TSource]:
+        return AsyncAnonymousObservable(self.subscribe_async)
+
+    def choose(self, chooser: Callable[[TSource], Option[TSource]]) -> AsyncObservable[TSource]:
+        """Choose.
+
+        Applies the given function to each element of the stream and returns
+        the stream comprised of the results for each element where the
+        function returns Some with some value.
+
+        Args:
+            chooser: A function to transform or filter the stream
+                by returning `Some(value)` or `Nothing`.
+
+        Returns:
+            The filtered and/or transformed stream.
+        """
+        return AsyncRx(pipe(self, choose(chooser)))
+
+    def choose_async(self, chooser: Callable[[TSource], Awaitable[Option[TSource]]]) -> AsyncObservable[TSource]:
+        """Choose async.
+
+        Applies the given async function to each element of the stream and
+        returns the stream comprised of the results for each element where
+        the function returns Some with some value.
+
+        Args:
+            chooser: A function to transform or filter the stream
+                asynchronously by returning `Some(value)` or `Nothing`.
+
+        Returns:
+            The filtered and transformed stream.
+        """
+        return AsyncRx(pipe(self, choose_async(chooser)))
+
     def combine_latest(self, other: TOther) -> "AsyncRx[Tuple[TSource, TOther]]":
         from .combine import combine_latest
 
-        return pipe(self, combine_latest(other), AsyncRx.create)
+        return AsyncRx.create(pipe(self, combine_latest(other)))
 
     def concat(self, other: AsyncObservable[TSource]) -> AsyncObservable[TSource]:
         from .combine import concat_seq
@@ -132,17 +188,47 @@ class AsyncRx(AsyncObservable[TSource]):
         return AsyncRx(delay(seconds)(self))
 
     def distinct_until_changed(self) -> AsyncObservable[TSource]:
-        from .filter import distinct_until_changed
+        from .filtering import distinct_until_changed
 
         return AsyncRx(distinct_until_changed(self))
 
     def filter(self, predicate: Callable[[TSource], bool]) -> "AsyncRx[TSource]":
-        from .filter import filter
+        """Filter stream.
 
-        return AsyncRx(pipe(self, filter(predicate)))
+        Filters the elements of an observable sequence based on a predicate.
+        Returns an observable sequence that contains elements from the input
+        sequence that satisfy the condition.
+
+        Args:
+            predicate:
+                A function to filter the stream by returning `True` to
+                keep the item, or `False` to filter and remove the item.
+
+        Returns:
+            The filtered stream.
+        """
+
+        from .filtering import filter as _filter
+
+        return AsyncRx(pipe(self, _filter(predicate)))
+
+    def filteri(self, predicate: Callable[[TSource, int], bool]) -> AsyncObservable[TSource]:
+        """Filter with index.
+
+        Filters the elements of an observable sequence based on a predicate
+        and incorporating the element's index on each element of the source.
+
+        Args:
+            predicate: Function to test each element.
+
+        Returns:
+            An observable sequence that contains elements from the input
+            sequence that satisfy the condition.
+        """
+        return AsyncRx(pipe(self, filteri(predicate)))
 
     def filter_async(self, predicate: Callable[[TSource], Awaitable[bool]]) -> "AsyncRx[TSource]":
-        from .filter import filter_async
+        from .filtering import filter_async
 
         return AsyncRx(pipe(self, filter_async(predicate)))
 
@@ -156,6 +242,24 @@ class AsyncRx(AsyncObservable[TSource]):
 
         return pipe(self, flat_map_async(selector), AsyncRx.create)
 
+    def flat_map_latest_async(
+        self, mapper: Callable[[TSource], Awaitable[AsyncObservable[TResult]]]
+    ) -> "AsyncRx[TResult]":
+        """Flat map latest async.
+
+        Asynchronosly transforms the items emitted by an source sequence
+        into observable streams, and mirror those items emitted by the
+        most-recently transformed observable sequence.
+
+        Args:
+            mapper (Callable[[TSource]): [description]
+            Awaitable ([type]): [description]
+
+        Returns:
+            Stream[TSource, TResult]: [description]
+        """
+        return AsyncRx(pipe(self, flat_map_latest_async(mapper)))
+
     def map(self, selector: Callable[[TSource], TResult]) -> "AsyncRx[TResult]":
         from .transform import map
 
@@ -180,8 +284,6 @@ class AsyncRx(AsyncObservable[TSource]):
         Returns:
             Stream[TSource, TSource]: [description]
         """
-        from .filter import skip
-
         return AsyncRx(pipe(self, skip(count)))
 
     def starfilter(self, predicate: Callable[..., bool]) -> AsyncObservable[Tuple[TSource, int]]:
@@ -192,17 +294,15 @@ class AsyncRx(AsyncObservable[TSource]):
             An observable sequence that contains elements from the input
             sequence that satisfy the condition.
         """
-        from .filter import starfilter
-
-        return AsyncRx(pipe(self, starfilter(predicate)))
+        return AsyncRx.create(pipe(self, starfilter(predicate)))
 
     def starmap(self, mapper: Callable[..., TResult]) -> AsyncObservable[TResult]:
         """Map and spread the arguments to the mapper.
 
-        Returns an observable sequence whose elements are the result of
-        invoking the mapper function on each element of the source."""
-
-        from .transform import starmap
+        Returns:
+            An observable sequence whose elements are the result of
+            invoking the mapper function on each element of the source.
+        """
 
         return AsyncRx(pipe(self, starmap(mapper)))
 
@@ -218,7 +318,7 @@ class AsyncRx(AsyncObservable[TSource]):
         Returns:
             Stream[TSource, TSource]: [description]
         """
-        from .filter import take
+        from .filtering import take
 
         return AsyncRx(pipe(self, take(count)))
 
@@ -234,7 +334,7 @@ class AsyncRx(AsyncObservable[TSource]):
         Returns:
             Stream[TSource, TSource]: [description]
         """
-        from .filter import take_last
+        from .filtering import take_last
 
         return AsyncRx(pipe(self, take_last(count)))
 
@@ -250,7 +350,7 @@ class AsyncRx(AsyncObservable[TSource]):
         Returns:
             Stream[TSource, TSource]: [description]
         """
-        from .filter import take_until
+        from .filtering import take_until
 
         return AsyncRx(pipe(self, take_until(other)))
 
@@ -262,17 +362,53 @@ class AsyncRx(AsyncObservable[TSource]):
     def with_latest_from(self, other: AsyncObservable[TOther]) -> "AsyncRx[Tuple[TSource, TOther]]":
         from .combine import with_latest_from
 
-        return AsyncRx(pipe(self, with_latest_from(other)))
+        return AsyncRx.create(pipe(self, with_latest_from(other)))
+
+
+def as_async_observable(source: AsyncObservable[TSource]) -> AsyncObservable[TSource]:
+    return AsyncAnonymousObservable(source.subscribe_async)
+
+
+def as_chained(source: AsyncObservable[TSource]) -> AsyncRx[TSource]:
+    return AsyncRx(source)
 
 
 def choose(chooser: Callable[[TSource], Option[TSource]]) -> Stream[TSource, TSource]:
-    from .filter import choose
+    """Choose.
+
+    Applies the given function to each element of the stream and returns
+    the stream comprised of the results for each element where the
+    function returns Some with some value.
+
+    Args:
+        chooser: A function to transform or filter the stream
+            by returning `Some(value)` or `Nothing`.
+
+    Returns:
+        The filtered and/or transformed stream.
+    """
+
+    from .filtering import choose
 
     return choose(chooser)
 
 
 def choose_async(chooser: Callable[[TSource], Awaitable[Option[TSource]]]) -> Stream[TSource, TSource]:
-    from .filter import choose_async
+    """Choose async.
+
+    Applies the given async function to each element of the stream and
+    returns the stream comprised of the results for each element where
+    the function returns Some with some value.
+
+    Args:
+        chooser: An async function to transform or filter the stream
+            by returning `Some(value)` or `Nothing`.
+
+    Returns:
+        The filtered and/or transformed stream.
+    """
+
+    from .filtering import choose_async
 
     return choose_async(chooser)
 
@@ -290,13 +426,15 @@ def debounce(seconds: float) -> Stream[TSource, TSource]:
     value before seconds has elapsed.
 
     Example:
-    partial = debounce(5) # 5 seconds
+        >>> partial = debounce(5) # 5 seconds
 
-    Keyword arguments:
-    seconds -- Duration of the throttle period for each value
+    Args:
+        seconds: Duration of the throttle period for each value
 
-    Returns a partially applied function that takes a source stream to
-    debounce."""
+    Returns:
+        A partially applied debounce function that takes the source
+        observable to debounce.
+    """
 
     from .timeshift import debounce
 
@@ -343,6 +481,12 @@ def delay(seconds: float) -> Stream[TSource, TSource]:
     return delay(seconds)
 
 
+def distinct_until_changed(source: AsyncObservable[TSource]) -> AsyncObservable[TSource]:
+    from .filtering import distinct_until_changed
+
+    return distinct_until_changed(source)
+
+
 def empty() -> "AsyncObservable[TSource]":
     from .create import empty
 
@@ -350,15 +494,47 @@ def empty() -> "AsyncObservable[TSource]":
 
 
 def filter(predicate: Callable[[TSource], bool]) -> Callable[[AsyncObservable[TSource]], AsyncObservable[TSource]]:
-    from .filter import filter
+    """Filter stream.
 
-    return filter(predicate)
+    Filters the elements of an observable sequence based on a predicate.
+    Returns an observable sequence that contains elements from the input
+    sequence that satisfy the condition.
+
+    Args:
+        predicate:
+            A function to filter the stream by returning `True` to
+            keep the item, or `False` to filter and remove the item.
+
+    Returns:
+        The filtered stream.
+    """
+    from .filtering import filter as _filter
+
+    return _filter(predicate)
+
+
+def filteri(predicate: Callable[[TSource, int], bool]) -> Stream[TSource, TSource]:
+    """Filter with index.
+
+    Filters the elements of an observable sequence based on a predicate
+    and incorporating the element's index on each element of the source.
+
+    Args:
+        predicate: Function to test each element.
+
+    Returns:
+        An observable sequence that contains elements from the input
+        sequence that satisfy the condition.
+    """
+    from .filtering import filteri
+
+    return filteri(predicate)
 
 
 def filter_async(
     predicate: Callable[[TSource], Awaitable[bool]]
 ) -> Callable[[AsyncObservable[TSource]], AsyncObservable[TSource]]:
-    from .filter import filter_async
+    from .filtering import filter_async
 
     return filter_async(predicate)
 
@@ -372,10 +548,13 @@ def from_async(worker: Awaitable[TSource]) -> AsyncObservable[TSource]:
 def from_iterable(iterable: Iterable[TSource]) -> AsyncObservable[TSource]:
     """Convert an iterable to a source stream.
 
-    1 - xs = from_iterable([1,2,3])
+    Example:
+        >>> xs = from_iterable([1,2,3])
 
-    Returns the source stream whose elements are pulled from the given
-    (async) iterable sequence."""
+    Returns:
+        The source stream whose elements are pulled from the given
+        (async) iterable sequence.
+    """
     from .create import of_seq
 
     return of_seq(iterable)
@@ -413,10 +592,29 @@ def flat_map_async(mapper: Callable[[TSource], Awaitable[AsyncObservable[TResult
     return flat_map_async(mapper)
 
 
+def flat_map_latest_async(mapper: Callable[[TSource], Awaitable[AsyncObservable[TResult]]]) -> Stream[TSource, TResult]:
+    """Flat map latest async.
+
+    Asynchronosly transforms the items emitted by an source sequence
+    into observable streams, and mirror those items emitted by the
+    most-recently transformed observable sequence.
+
+    Args:
+        mapper (Callable[[TSource]): [description]
+        Awaitable ([type]): [description]
+
+    Returns:
+        Stream[TSource, TResult]: [description]
+    """
+    from .transform import flat_map_latest_async
+
+    return flat_map_latest_async(mapper)
+
+
 def from_async_iterable(iter: Iterable[TSource]) -> "AsyncObservable[TSource]":
     from aioreactive.operators.from_async_iterable import from_async_iterable
 
-    from .create import of
+    from .create import from_async_iterable
 
     return AsyncRx(from_async_iterable(iter))
 
@@ -465,15 +663,6 @@ def mapi(mapper: Callable[[TSource, int], TResult]) -> Stream[TSource, TResult]:
     return mapi(mapper)
 
 
-def merge_inner(max_concurrent: int = 0) -> Stream[AsyncObservable[TSource], TSource]:
-    def _merge_inner(source: AsyncObservable[AsyncObservable[TSource]]) -> AsyncObservable[TSource]:
-        from .combine import merge_inner
-
-        return pipe(source, merge_inner(max_concurrent))
-
-    return _merge_inner
-
-
 def merge(other: AsyncObservable[TSource]) -> Stream[TSource, TSource]:
     from .create import of_seq
 
@@ -483,6 +672,15 @@ def merge(other: AsyncObservable[TSource]) -> Stream[TSource, TSource]:
     return _
 
 
+def merge_inner(max_concurrent: int = 0) -> Stream[AsyncObservable[TSource], TSource]:
+    def _merge_inner(source: AsyncObservable[AsyncObservable[TSource]]) -> AsyncObservable[TSource]:
+        from .combine import merge_inner
+
+        return pipe(source, merge_inner(max_concurrent))
+
+    return _merge_inner
+
+
 def merge_seq(sources: Iterable[AsyncObservable[TSource]]) -> AsyncObservable[TSource]:
     from .create import of_seq
 
@@ -495,14 +693,10 @@ def never() -> "AsyncObservable[TSource]":
     return never()
 
 
-def distinct_until_changed(source: AsyncObservable[TSource]) -> AsyncObservable[TSource]:
-    from .filter import distinct_until_changed
+def of_async(workflow: Awaitable[TSource]) -> AsyncObservable[TSource]:
+    from .create import of_async
 
-    return distinct_until_changed(source)
-
-
-def as_chained(source: AsyncObservable[TSource]) -> AsyncRx[TSource]:
-    return AsyncRx(source)
+    return of_async(workflow)
 
 
 def retry(retry_count: int) -> Stream[TSource, TSource]:
@@ -522,7 +716,54 @@ def subscribe_async(obv: AsyncObserver[TSource]) -> Callable[[AsyncObservable[TS
     return subscribe_async(obv)
 
 
-def switch_latest() -> Stream[TSource, TSource]:
+def single(value: TSource) -> "AsyncObservable[TSource]":
+    from .create import single
+
+    return single(value)
+
+
+def skip(count: int) -> Stream[TSource, TSource]:
+    """Skip items in the stream.
+
+    Bypasses a specified number of elements in an observable sequence
+    and then returns the remaining elements.
+
+    Args:
+        count (int): Items to skip
+
+    Returns:
+        The result stream with skipped items.
+    """
+    from .filtering import skip
+
+    return skip(count)
+
+
+def starfilter(predicate: Callable[..., bool]) -> Stream[TSource, Tuple[Any, ...]]:
+    """Filter and spread the arguments to the predicate.
+
+    Filters the elements of an observable sequence based on a predicate.
+    Returns:
+        An observable sequence that contains elements from the input
+        sequence that satisfy the condition.
+    """
+    from .filtering import starfilter
+
+    return starfilter(predicate)
+
+
+def starmap(mapper: Callable[..., TResult]) -> Stream[TSource, TResult]:
+    """Map and spread the arguments to the mapper.
+
+    Returns an observable sequence whose elements are the result of
+    invoking the mapper function on each element of the source."""
+
+    from .transform import starmap
+
+    return starmap(mapper)
+
+
+def switch_latest() -> Stream[AsyncObservable[TSource], TSource]:
     from .transform import switch_latest
 
     return switch_latest
@@ -540,7 +781,7 @@ def take(count: int) -> Stream[TSource, TSource]:
     Returns:
         Stream[TSource, TSource]: [description]
     """
-    from .filter import take
+    from .filtering import take
 
     return take(count)
 
@@ -557,7 +798,7 @@ def take_last(count: int) -> Stream[TSource, TSource]:
     Returns:
         Stream[TSource, TSource]: [description]
     """
-    from .filter import take_last
+    from .filtering import take_last
 
     return take_last(count)
 
@@ -574,72 +815,26 @@ def take_until(other: AsyncObservable[TResult]) -> Stream[TSource, TSource]:
     Returns:
         Stream[TSource, TSource]: [description]
     """
-    from .filter import take_until
+    from .filtering import take_until
 
     return take_until(other)
 
 
+def timer(due_time: float) -> AsyncObservable[int]:
+    """Returns an observable sequence that triggers the value 0
+    after the given duetime in milliseconds.
+    """
+    from .create import timer
+
+    return timer(due_time)
+
+
 def to_async_iterable(source: AsyncObservable[TSource]) -> AsyncIterable[TSource]:
     from .leave import to_async_iterable
 
     return to_async_iterable(source)
 
 
-def single(value: TSource) -> "AsyncObservable[TSource]":
-    from .create import single
-
-    return single(value)
-
-
-def skip(count: int) -> Stream[TSource, TSource]:
-    """Skip items in the stream.
-
-    Bypasses a specified number of elements in an observable sequence
-    and then returns the remaining elements.
-
-    Args:
-        count (int): Items to skip
-
-    Returns:
-        Stream[TSource, TSource]: [description]
-    """
-    from .filter import skip
-
-    return skip(count)
-
-
-def starfilter(predicate: Callable[..., bool]) -> Stream[TSource, Tuple[TSource, int]]:
-    """Filter and spread the arguments to the predicate.
-
-    Filters the elements of an observable sequence based on a predicate.
-    Returns:
-        An observable sequence that contains elements from the input
-        sequence that satisfy the condition.
-    """
-    from .filter import starfilter
-
-    return starfilter(predicate)
-
-
-def starmap(mapper: Callable[..., TResult]) -> Stream[TSource, TResult]:
-    """Map and spread the arguments to the mapper.
-
-    Returns an observable sequence whose elements are the result of
-    invoking the mapper function on each element of the source."""
-
-    from .transform import starmap
-
-    return starmap(mapper)
-
-
-def timer(due_time: float) -> AsyncObservable[int]:
-    """Returns an observable sequence that triggers the value 0
-    after the given duetime in milliseconds."""
-    from .create import timer
-
-    return timer(due_time)
-
-
 def with_latest_from(other: AsyncObservable[TOther]) -> Stream[TSource, Tuple[TSource, TOther]]:
     from .combine import with_latest_from
 
@@ -666,9 +861,15 @@ __all__ = [
     "concat_seq" "delay",
     "empty",
     "filter",
-    "filteri" "filter_async",
+    "filteri",
+    "filter_async",
     "from_async",
     "from_iterable",
+    "flat_map",
+    "flat_mapi",
+    "flat_map_async",
+    "flat_mapi_async",
+    "flat_map_latest_async",
     "map",
     "map_async",
     "merge",
@@ -685,7 +886,10 @@ __all__ = [
     "to_async_iterable",
 ]
 
+
+# flake8: noqa
 from ._version import get_versions
 
-__version__ = get_versions()["version"]
+__version__ = get_versions()["version"]  # type: ignore
+
 del get_versions
